import 'dart:io';

import 'package:camera/camera.dart';
import 'package:camera_platform_interface/camera_platform_interface.dart';
import 'package:flutter/material.dart';
import 'package:image/image.dart' as imglib;
import 'package:kyc/Components/Navigator/back_pop_up.dart';
import 'package:kyc/Components/Preview/document_preview.dart';
import 'package:kyc/Components/document_type.dart';
import 'package:kyc/Components/state.dart';
import 'package:kyc/Components/step.dart';
import 'package:path_provider/path_provider.dart';
import 'package:provider/provider.dart';

/*
TODO
 FACTORISE FACE & DOCUMENT SCAN
 EXPLODE THE HUGE COMPONENT INTO SMALLER ONES
 ADD EDGE DETECTION
 TRIGGER PICTURE TAKING ON EDGE DETECTION
 HANDLE BACK BUTTON TO GO TO THE BEGINING
*/
class ScanDocumentStep extends StatefulWidget {
  static const routeName = "/scan_document";
  final DocumentType type;
  final KycStep step;

  ScanDocumentStep({key, this.step, this.type}) : super(key: key);

  @override
  _ScanDocumentStepState createState() => _ScanDocumentStepState();
}

class _ScanDocumentStepState extends State<ScanDocumentStep>
    with WidgetsBindingObserver {
  CameraController _cameraController;
  bool _isPhysicalDevice = false;
  BoxBorder _sightBorder = Border.all(width: 5, color: Colors.white);
  String _stepInstructions = "";
  String _stepLabel = "";
  String _title = "";

  @override
  initState() {
    super.initState();
    _prepareInstructions();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    final CameraController cameraController = _cameraController;

    // App state changed before we got the chance to initialize.
    if (cameraController == null || !cameraController.value.isInitialized) {
      return;
    }

    if (state == AppLifecycleState.inactive) {
      cameraController.dispose();
    } else if (state == AppLifecycleState.resumed) {
      cameraController.addListener(() {
        if (mounted) setState(() {});
      });
    }
  }

  _prepareInstructions() {
    switch (widget.type) {
      case DocumentType.IdentityCard:
        this._title = "Identity Card";
        break;
      case DocumentType.DriversLicense:
        this._title = "Driver's License";
        break;
      case DocumentType.Passport:
        this._title = "Passport";
        break;
    }

    switch (widget.step) {
      case KycStep.Recto:
        this._stepLabel = "Front of your document";
        this._stepInstructions =
            "Position the front of your document in the frame";
        break;
      case KycStep.Verso:
        this._stepLabel = "Back of your document";
        this._stepInstructions =
            "Position the back of your document in the frame";
        break;
    }
  }

  @override
  dispose() {
    super.dispose();
  }

  _takePicture(BuildContext context) async {
    final CameraController cameraController = _cameraController;
    Image img;
    File croppedFile;
    XFile imageFile = await cameraController.takePicture();
    imglib.Image picture = imglib.decodeJpg(await imageFile.readAsBytes());
    int width = picture.width;
    int height = picture.height;
    int sightHeight = (height * 0.4).toInt();
    int sightWidth = (width * 0.93).toInt();
    var sideOffset = (width * 0.03).toInt();
    var topOffset = (height * 0.14).toInt();
    imglib.Image croppedPicture = imglib.copyCrop(
        picture, sideOffset, topOffset, sightWidth, sightHeight);
    Directory dir = await getApplicationSupportDirectory();
    String fileName = dir.path +
        "/" +
        DateTime.now().toString() +
        widget.step.toString() +
        ".jpg";
    croppedFile = File(fileName);
    var jpgPicture = imglib.encodeJpg(croppedPicture);
    croppedFile = await croppedFile.writeAsBytes(jpgPicture);
    img = Image.memory(jpgPicture, fit: BoxFit.cover);
    _redirect(context, img, croppedFile.path);
  }

  void _redirect(BuildContext context, Image img, String imagePath) {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (context) =>
                DocumentPreview(img: img, imgPath: imagePath)));
  }

  void _redirectSimulator(BuildContext context) {
    Widget redirection = context.read<KycState>().getSimulatorPreview();
    String path = context.read<KycState>().getSimulatorPreviewPath();
    this._redirect(context, redirection, path);
  }

  _buildTakePictureButton(BuildContext context) {
    bool isPhysicalDevice = context.read<KycState>().isPhysicalDevice();
    return FloatingActionButton(
        child: Icon(Icons.photo_camera_rounded),
        onPressed: () => isPhysicalDevice
            ? _takePicture(context)
            : _redirectSimulator(context));
  }

  _buildSight(BuildContext context) {
    double height = MediaQuery.of(context).size.height;
    double width = MediaQuery.of(context).size.width;

    double sightHeight = height * 0.35;

    var sideOffset = (width * 0.05);

    return Stack(
      children: [
        ColorFiltered(
            colorFilter: ColorFilter.mode(
                Colors.black.withOpacity(0.8), BlendMode.srcOut),
            child: Stack(
              fit: StackFit.expand,
              children: [
                Container(
                  decoration: BoxDecoration(
                      color: Colors.black,
                      backgroundBlendMode: BlendMode.dstOut),
                ),
                Positioned(
                    top: height * 0.12,
                    left: sideOffset / 2,
                    child: Container(
                        width: width - sideOffset,
                        height: sightHeight,
                        decoration: BoxDecoration(
                          color: Colors.black,
                          shape: BoxShape.rectangle,
                        ))),
              ],
            )),
        Positioned(
            top: height * 0.12,
            left: sideOffset / 2,
            child: Container(
              decoration: BoxDecoration(border: _sightBorder),
              width: width - sideOffset,
              height: sightHeight,
            )),
        Positioned(
            top: height * 0.2,
            left: sideOffset / 2,
            child: Container(
              decoration: BoxDecoration(
                  border: Border.symmetric(
                      vertical: BorderSide(
                          width: 5, color: Colors.black.withOpacity(0.7)))),
              width: width - sideOffset,
              height: sightHeight / 2,
            )),
        Positioned(
            top: height * 0.12,
            left: sideOffset * 4,
            child: Container(
              decoration: BoxDecoration(
                  border: Border.symmetric(
                      horizontal: BorderSide(
                          width: 5, color: Colors.black.withOpacity(0.7)))),
              width: width - sideOffset * 8,
              height: sightHeight,
            )),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    double height = MediaQuery.of(context).size.height;
    double width = MediaQuery.of(context).size.width;
    this._cameraController = context.read<KycState>().cameraController;

    var preview = context.read<KycState>().getPreview();
    this._isPhysicalDevice = context.read<KycState>().isPhysicalDevice();
    if (!_isPhysicalDevice) {
      preview = Positioned(child: preview, top: height * 0.12, width: width);
    }
    return WillPopScope(
      onWillPop: () {
        BackPopup.standardPopup(context);
        return;
      },
      child: Scaffold(
        appBar: AppBar(title: Text("Identity Validation")),
        body: Center(
            child: SizedBox.expand(
          child: Stack(
            fit: StackFit.expand,
            children: [
              preview,
              _buildSight(context),
              Positioned(
                  top: height * 0.01,
                  left: height * 0.01,
                  child: Row(
                    children: [
                      Text(
                        _title,
                        style: TextStyle(
                            fontSize: height * 0.04, color: Colors.white),
                      )
                    ],
                  )),
              Positioned(
                  width: width,
                  bottom: height * 0.3,
                  child: Center(
                      child: Text(
                    this._stepLabel,
                    style:
                        TextStyle(fontSize: height * 0.03, color: Colors.white),
                    textWidthBasis: TextWidthBasis.parent,
                  ))),
              Positioned(
                  width: width,
                  bottom: height * 0.27,
                  child: Center(
                      child: Text(
                    this._stepInstructions,
                    style:
                        TextStyle(fontSize: height * 0.02, color: Colors.white),
                    textWidthBasis: TextWidthBasis.parent,
                  ))),
            ],
          ),
        )),
        floatingActionButton: _buildTakePictureButton(context),
        floatingActionButtonLocation: FloatingActionButtonLocation.centerDocked,
      ),
    );
  }
}
