import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import 'package:kyc/Components/FaceScan/face_scan.dart';
import 'package:kyc/Components/Transitions/end_preview.dart';
import 'package:kyc/Components/document_type.dart';
import 'package:kyc/Exceptions/api_exceptions.dart';
import 'package:kyc/Exceptions/sdk_exceptions.dart';
import 'package:kyc/Models/kyc_document.dart';
import 'package:kyc/Services/document_service.dart';
import 'package:provider/src/provider.dart';

import '../state.dart';

class LoadingScreen extends StatefulWidget {
  final KycDocument document;

  const LoadingScreen({Key key, this.document}) : super(key: key);

  @override
  _LoadingScreenState createState() => _LoadingScreenState();
}

class _LoadingScreenState extends State<LoadingScreen> {
  DocumentService _documentService = new DocumentService();

  @override
  Widget build(BuildContext context) {
    double height = MediaQuery.of(context).size.height;
    return Scaffold(
        appBar: AppBar(title: Text("Sending document")),
        body: Stack(
          fit: StackFit.expand,
          children: [
            Center(
              child: Container(
                  child: CircularProgressIndicator(
                valueColor: new AlwaysStoppedAnimation<Color>(Colors.blue),
              )),
            ),
            Padding(
              padding: const EdgeInsets.only(top: 20.0),
            ),
            Padding(
              padding: const EdgeInsets.only(top: 100),
              child: Center(
                child: Text("Sending Document",
                    style: TextStyle(
                      color: Colors.blue,
                      fontSize: height * 0.025,
                      fontWeight: FontWeight.bold,
                    )),
              ),
            )
          ],
        ));
  }

  @override
  void initState() {
    super.initState();
    SchedulerBinding.instance.addPostFrameCallback((_) {
      runInitTasks();
    });
  }

  @protected
  Future runInitTasks() async {
    bool done = false;
    bool isPhysicalDevice = context.read<KycState>().isPhysicalDevice();
    var country = context.read<KycState>().getCountry().trigram;
    bool error = false;
    done = widget.document != null &&
        widget.document.type == DocumentType.FaceScan;
    String message = "Done";
    dynamic response;
    try {
      response = await _documentService.sendDocument(widget.document, country);
      message = "response : $response";
    } catch (e) {
      if (e is SdkException) {
        if (e is ApiException) {
          print(e.toString());
        }
        message =
            "Error while sending documents with code error : ${e.errorCode}";
        done = false;
        error = true;
      }
      print(message);
    }

    done = done || context.read<KycState>().isProcessComplete();
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (context) => done ? EndPreviewScreen() : FaceScanStep()));
  }
}
