import 'package:flutter/material.dart';
import 'package:kyc/Components/IdScan/scan_document.dart';
import 'package:kyc/Components/Loading/loading_screen.dart';
import 'package:kyc/Components/Navigator/back_pop_up.dart';
import 'package:kyc/Components/document_type.dart';
import 'package:kyc/Components/state.dart';
import 'package:kyc/Components/step.dart';
import 'package:kyc/Models/kyc_document.dart';
import 'package:provider/provider.dart';

class DocumentPreview extends StatefulWidget {
  final Image img;
  final String imgPath;
  static const routeName = "/document_preview";

  const DocumentPreview({Key key, this.img, this.imgPath}) : super(key: key);

  @override
  _DocumentPreviewState createState() => _DocumentPreviewState();
}

class _DocumentPreviewState extends State<DocumentPreview> {
  static final String TITLE = "Document Preview";
  static final String CLEARDIL_ICON_PATH = "assets/images/cleardil-icon.png";
  static final String CLEARDIL_BRAND_NAME = "Cleardil™";
  static final String STEP_INDICATIONS = "Make sure your details are clear";

  @override
  Widget build(BuildContext context) {
    double height = MediaQuery.of(context).size.height;
    double width = MediaQuery.of(context).size.width;
    var indicationTop = height / 2.2;
    var indicationLeft = width / 6;
    var iconTop = height * 0.05;
    if (context.read<KycState>().isProcessComplete()) {
      indicationTop = height / 1.5;
      indicationLeft = width / 5;
      iconTop = height * 0.015;
    }

    return WillPopScope(
      onWillPop: () {
        BackPopup.standardPopup(context);
        return;
      },
      child: Scaffold(
          appBar: AppBar(title: Text(TITLE)),
          body: Stack(
            fit: StackFit.expand,
            children: [
              Positioned(
                  left: width / 2.5,
                  top: iconTop,
                  child: Container(
                      child: Row(
                    children: [
                      Image.asset(
                        CLEARDIL_ICON_PATH,
                        scale: 3.5,
                      ),
                      Padding(
                        padding: EdgeInsets.only(left: width * 0.01),
                        child: Text(
                          CLEARDIL_BRAND_NAME,
                          style: TextStyle(fontSize: height * 0.017),
                        ),
                      )
                    ],
                  ))),
              _buildPreview(context, width, height),
              Positioned(
                  top: indicationTop,
                  left: indicationLeft,
                  child: Container(
                      width: height * 0.9,
                      child: Text(
                        STEP_INDICATIONS,
                        style: TextStyle(fontSize: height * 0.023),
                      ))),
              Align(
                alignment: Alignment.bottomCenter,
                child: ButtonBar(
                  alignment: MainAxisAlignment.center,
                  children: [
                    TextButton(
                      onPressed: () {
                        _handleValidatePicture(context);
                      },
                      child: Text(
                        _getTextValidate(context),
                        style: TextStyle(
                            color: Colors.white, fontSize: height * 0.025),
                      ),
                      style: ButtonStyle(
                          maximumSize: MaterialStateProperty.all(
                              Size.fromWidth(width * 0.89)),
                          padding: MaterialStateProperty.all(
                              EdgeInsets.symmetric(horizontal: width * 0.27)),
                          backgroundColor:
                              MaterialStateProperty.all(Colors.blue),
                          foregroundColor:
                              MaterialStateProperty.all(Colors.blue)),
                    ),
                    OutlinedButton(
                        onPressed: () {
                          Navigator.pop(context);
                        },
                        child: Text(
                          "Take new picture",
                          style: TextStyle(
                              color: Colors.blue, fontSize: height * 0.025),
                        ),
                        style: ButtonStyle(
                          maximumSize: MaterialStateProperty.all(
                              Size.fromWidth(width * 0.9)),
                          padding: MaterialStateProperty.all(
                              EdgeInsets.symmetric(horizontal: width * 0.25)),
                        ))
                  ],
                ),
              )
            ],
          )),
    );
  }

  String _getTextValidate(BuildContext context) {
    KycDocument document = context.read<KycState>().document;
    DocumentType type = context.read<KycState>().getDocumentType();
    if (type == DocumentType.FaceScan) {
      return "Send Picture";
    } else if (type == DocumentType.IdentityCard &&
        document.frontSide == null) {
      return "Validate picture";
    } else {
      return "Send document";
    }
  }

  void _handleValidatePicture(BuildContext context) async {
    KycDocument document = context.read<KycState>().document;
    DocumentType type = context.read<KycState>().getDocumentType();
    if (type == DocumentType.IdentityCard && document.frontSide == null) {
      document.frontSide = widget.imgPath;
      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (context) => ScanDocumentStep(
                    type: DocumentType.IdentityCard,
                    step: KycStep.Verso,
                  )));
    } else {
      if (document.type == DocumentType.FaceScan) {
        document.selphie = widget.imgPath;
      } else if (type == DocumentType.IdentityCard) {
        document.backSide = widget.imgPath;
      } else {
        document.frontSide = widget.imgPath;
      }
      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (context) => LoadingScreen(document: document)));
    }
  }

  _buildPreview(BuildContext context, double width, double height) {
    var documentPreview = Align(
      alignment: Alignment.topCenter,
      child: Padding(
        padding: EdgeInsets.only(top: height * 0.1),
        child: Container(
          width: width * 0.9,
          child: widget.img,
        ),
      ),
    );
    var facePreview = Align(
        alignment: Alignment.topCenter,
        child: Padding(
          padding: EdgeInsets.only(top: height * 0.055),
          child:
              Container(height: height * 0.6, width: width, child: widget.img),
        ));
    if (context.read<KycState>().isProcessComplete()) {
      return facePreview;
    } else {
      return documentPreview;
    }
  }
}
