import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:kyc/Components/IdScan/scan_document.dart';
import 'package:kyc/Components/Navigator/back_pop_up.dart';
import 'package:kyc/Components/Transitions/search_countries.dart';
import 'package:kyc/Models/country.dart';
import 'package:kyc/Services/country_service.dart';
import 'package:provider/provider.dart';

import '../state.dart';
import '../step.dart';

class CountrySelector extends StatefulWidget {
  @override
  _CountrySelectorState createState() => _CountrySelectorState();
}

class _CountrySelectorState extends State<CountrySelector> {
  List<Country> _countries = List.empty(growable: true);
  double _height;
  double _width;
  static const TITLE = "Choose your type of document";

  Future<List<dynamic>> _listCountries() async {
    CountryService srvc = CountryService();
    var response = await srvc.getCountryList();
    this._countries = response;
    return _countries;
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
        future: _listCountries(),
        builder: (BuildContext context, AsyncSnapshot<List<dynamic>> snapshot) {
          return WillPopScope(
            onWillPop: () {
              BackPopup.standardPopup(context);
              return;
            },
            child: Scaffold(
                appBar: _buildAppBar(context),
                body: ListView.builder(
                    scrollDirection: Axis.vertical,
                    itemCount: _countries.length,
                    itemBuilder: (context, index) {
                      return ListTile(
                          onTap: () => _handleCountrySelection(
                              context, snapshot.data[index]),
                          title: Text(snapshot.data[index].label));
                    })),
          );
        });
  }

  _buildAppBar(BuildContext context) {
    return AppBar(
      actions: [
        IconButton(
            onPressed: () => Navigator.of(context).push(MaterialPageRoute(
                builder: (_) => SearchPage(countries: _countries))),
            icon: Icon(Icons.search))
      ],
      title: Text("Select your country"),
    );
  }

  _handleCountrySelection(BuildContext context, Country country) {
    context.read<KycState>().setCountry(country);
    _handleRedirect(context);
  }

  _handleRedirect(BuildContext context) {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (context) => ScanDocumentStep(
                  type: context.read<KycState>().getDocumentType(),
                  step: KycStep.Recto,
                )));
  }
}
