import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:kyc/Components/document_type.dart';
import 'package:kyc/Components/state.dart';
import 'package:kyc/Utils/channel_utils.dart';
import 'package:provider/provider.dart';

import 'country_selector.dart';

class DocumentSelector extends StatefulWidget {
  @override
  _DocumentSelectorState createState() => _DocumentSelectorState();
}

class _DocumentSelectorState extends State<DocumentSelector> {
  double _height;
  double _width;
  static const TITLE = "Choose your type of document";

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<List<DocumentType>>(
        future: ChannelUtils.getListDocument(),
        builder:
            (BuildContext context, AsyncSnapshot<List<DocumentType>> snapshot) {
          if (snapshot.connectionState == ConnectionState.waiting) {
            return Center(child: Text('Please wait its loading...'));
          } else {
            if (snapshot.hasError) {
              return Center(child: Text('Error: ${snapshot.error}'));
            } else {
              _height = MediaQuery.of(context).size.height;
              _width = MediaQuery.of(context).size.width;

              List<Widget> widgets = getWidgetFromTypes(snapshot.data);

              return Scaffold(
                appBar: AppBar(title: Text("Cleardil KYC")),
                body: Center(
                  child: Container(
                    child: Column(children: [
                      Padding(
                        padding: EdgeInsets.only(
                            top: _height * 0.15, bottom: _height * 0.05),
                        child: Text(
                          TITLE,
                          style: TextStyle(fontSize: _height * 0.025),
                        ),
                      ),
                      Container(
                          constraints:
                              BoxConstraints.tightFor(width: _width * 0.7),
                          child: Column(
                            children: widgets,
                          )),
                    ]),
                  ),
                ),
              );
            }
          }
        });
  }

  List<Widget> getWidgetFromTypes(List<DocumentType> types) {
    if (types.isEmpty) {
      types.add(DocumentType.IdentityCard);
      types.add(DocumentType.Passport);
      types.add(DocumentType.DriversLicense);
    }
    var widgets = List<Widget>.empty(growable: true);

    types.forEach((element) {
      switch (element) {
        case DocumentType.IdentityCard:
          widgets.add(buildButton(context, DocumentType.IdentityCard));
          break;
        case DocumentType.Passport:
          widgets.add(buildButton(context, DocumentType.Passport));
          break;
        case DocumentType.DriversLicense:
          widgets.add(buildButton(context, DocumentType.DriversLicense));
          break;
      }
    });

    return widgets;
  }

  Widget buildButton(BuildContext context, DocumentType type) {
    bool rotate = false;
    String buttonLabel = "";
    IconData icon = Icons.not_interested;
    double titlePaddingLeft,
        titlePaddingRight,
        titlePaddingTop,
        titlePaddingBottom;
    double iconPaddingLeft, iconPaddingRight, iconPaddingTop, iconPaddingBottom;
    titlePaddingLeft = _width * 0.05;
    titlePaddingRight = _width * 0.05;
    titlePaddingTop = _height * 0.01;
    titlePaddingBottom = _height * 0.01;
    switch (type) {
      case DocumentType.IdentityCard:
        buttonLabel = "Identity Card";
        icon = Icons.perm_identity;
        break;
      case DocumentType.DriversLicense:
        buttonLabel = "Driver's license";
        icon = Icons.directions_car_rounded;
        break;
      case DocumentType.Passport:
        buttonLabel = "Passport";
        icon = Icons.airplanemode_on;
        rotate = true;
        titlePaddingLeft = 0;
        titlePaddingRight = _width * 0.11;
        titlePaddingTop = _height * 0.01;
        titlePaddingBottom = _height * 0.01;
        break;
      default:
    }

    return Padding(
      padding: EdgeInsets.only(bottom: _height * 0.02),
      child: OutlinedButton(
          onPressed: () {
            _handleNavigation(context, type);
          },
          child: Row(mainAxisAlignment: MainAxisAlignment.center, children: [
            buildButtonIcon(icon, rotate),
            Padding(
              padding: EdgeInsets.only(
                  left: titlePaddingLeft,
                  right: titlePaddingRight,
                  top: titlePaddingTop,
                  bottom: titlePaddingBottom),
              child: Text(buttonLabel,
                  style: TextStyle(
                    color: Colors.blue,
                    fontSize: _height * 0.025,
                    fontWeight: FontWeight.bold,
                  )),
            )
          ]),
          style: ButtonStyle(
              backgroundColor: MaterialStateProperty.all(Colors.transparent))),
    );
  }

  Widget buildButtonIcon(IconData icon, bool rotate) {
    Widget widget = Icon(
      icon,
      color: Colors.blue,
    );
    if (rotate) {
      widget = Padding(
        padding: EdgeInsets.only(right: _width * 0.09),
        child: RotatedBox(quarterTurns: 3, child: widget),
      );
    }
    return widget;
  }

  void _handleNavigation(BuildContext context, DocumentType docType) {
    context.read<KycState>().reset();
    context.read<KycState>().setDocumentType(docType);
    Navigator.push(
        context, MaterialPageRoute(builder: (context) => CountrySelector()));
  }
}
