import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:kyc/Components/IdScan/scan_document.dart';
import 'package:kyc/Models/country.dart';
import 'package:provider/src/provider.dart';

import '../state.dart';
import '../step.dart';

class SearchPage extends StatefulWidget {
  final List<Country> countries;

  const SearchPage({Key key, this.countries}) : super(key: key);

  @override
  _SearchPageState createState() => _SearchPageState(countries);
}

class _SearchPageState extends State<SearchPage> {
  List<Country> countries;
  List<Country> cache;
  var _controller = TextEditingController();

  _SearchPageState(List<Country> countries) {
    this.countries = countries;
    this.cache = countries;
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          title: Container(
            width: double.infinity,
            height: 40,
            decoration: BoxDecoration(
                color: Colors.white, borderRadius: BorderRadius.circular(5)),
            child: Center(
              child: TextField(
                controller: _controller,
                onChanged: (String value) {
                  _handleOnChange(value);
                },
                decoration: InputDecoration(
                    prefixIcon: Icon(Icons.search),
                    suffixIcon: IconButton(
                      icon: Icon(Icons.clear),
                      onPressed: () {
                        _controller.clear();
                        setState(() => this.countries = this.cache);
                      },
                    ),
                    hintText: 'Search...',
                    border: InputBorder.none),
              ),
            ),
          ),
        ),
        body: ListView.builder(
            scrollDirection: Axis.vertical,
            itemCount: countries.length,
            itemBuilder: (context, index) {
              return Dismissible(
                  direction: DismissDirection.none,
                  key: Key(countries[index].label.toString()),
                  child: ListTile(
                      onTap: () =>
                          _handleCountrySelection(context, countries[index]),
                      title: Text(countries[index].label)));
            }));
  }

  _handleOnChange(String value) {
    if (value.isEmpty) {
      setState(() => this.countries = this.cache);
    }
    List<Country> countries = this
        .countries
        .where((element) =>
            element.label.toLowerCase().contains(value.toLowerCase()))
        .toList();
    setState(() => this.countries = countries);
  }

  _handleCountrySelection(BuildContext context, Country country) {
    context.read<KycState>().setCountry(country);
    _handleRedirect(context);
  }

  _handleRedirect(BuildContext context) {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (context) => ScanDocumentStep(
                  type: context.read<KycState>().getDocumentType(),
                  step: KycStep.Recto,
                )));
  }
}
