import 'package:camera/camera.dart';
import 'package:camera_platform_interface/camera_platform_interface.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:kyc/Components/document_type.dart';
import 'package:kyc/Models/country.dart';
import 'package:kyc/Models/kyc_document.dart';

class KycState extends ChangeNotifier {
  bool processDone = false;
  KycDocument document;
  bool physicalDevice = true;
  CameraController cameraController;
  Widget cameraPreview;
  static String SIMULATOR_FRONT_PREVIEW = "assets/images/sample_id_front.png";
  static String SIMULATOR_BACK_PREVIEW = "assets/images/sample_id_back.png";
  static String SIMULATOR_FACE_PREVIEW = "assets/images/face_sample.jpeg";
  Image simulatorFrontPreview = Image.asset(SIMULATOR_FRONT_PREVIEW);
  Image simulatorBackPreview = Image.asset(SIMULATOR_BACK_PREVIEW);
  Image simulatorFacePreview = Image.asset(SIMULATOR_FACE_PREVIEW);
  Country country;

  KycState.physical(this.cameraController) {
    this.physicalDevice = true;
    this.cameraPreview =
        CameraPlatform.instance.buildPreview(this.cameraController.cameraId);
  }

  KycState.simulator() {
    physicalDevice = false;
  }

  void reset() {
    document = new KycDocument();
  }

  bool isPhysicalDevice() {
    return physicalDevice;
  }

  DocumentType getDocumentType() {
    return this.document.type;
  }

  void setDocumentType(DocumentType type) {
    this.document.type = type;
  }

  Widget getSimulatorPreview() {
    if (document.frontSide == null) {
      return this.simulatorFrontPreview;
    } else {
      return this.simulatorBackPreview;
    }
  }

  Widget getFaceSimulatorPreview() {
    return simulatorFacePreview;
  }

  String getSimulatorFacePreviewPath() {
    return SIMULATOR_FACE_PREVIEW;
  }

  String getSimulatorPreviewPath() {
    return SIMULATOR_FRONT_PREVIEW;
  }

  void completeProcess() {
    this.processDone = true;
  }

  bool isProcessComplete() {
    return this.processDone;
  }

  Widget getPreview() {
    if (physicalDevice) {
      return this.cameraPreview;
    } else {
      return this.getSimulatorPreview();
    }
  }

  Widget getFacePreview() {
    if (physicalDevice) {
      return this.cameraPreview;
    } else {
      return this.getFaceSimulatorPreview();
    }
  }

  void setCountry(Country country) {
    this.country = country;
  }

  Country getCountry() {
    return this.country;
  }

  void setCameraPreview(CameraController controller) {
    this.cameraController = controller;
    this.cameraPreview =
        CameraPlatform.instance.buildPreview(cameraController.cameraId);
  }
}
