import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:http/http.dart' as http;
import 'package:http_parser/http_parser.dart';
import 'package:kyc/Components/document_type.dart';
import 'package:kyc/Exceptions/api_exceptions.dart';
import 'package:kyc/Helper/iapi_helper.dart';
import 'package:kyc/Models/kyc_document.dart';

class ApiHelper implements IApiHelper {
  final String _url;
  String _sdkToken;

  ApiHelper(this._url);

  @override
  void setSdkToken(String token) {
    _sdkToken = token;
  }

  Future<void> addToRequest(
      http.MultipartRequest request, String imageUri, String field) async {
    if (imageUri != null && imageUri.isNotEmpty) {
      http.MultipartFile file = await http.MultipartFile.fromPath(
          field, imageUri,
          contentType: MediaType('image', 'jpg'));
      request.files.add(file);
    }
  }

  @override
  Future<dynamic> uploadDocument(KycDocument document, String country) async {
    var responseJson;
    try {
      var request =
          new http.MultipartRequest("POST", Uri.parse(_url + "/sdk/documents"));
      request.headers["sdk-token"] = _sdkToken;
      await addToRequest(request, document.frontSide, "front_side");
      await addToRequest(request, document.backSide, "back_side");
      request.fields["type"] = document.type.str;
      request.fields["issuing_country"] = country;
      var streamedResponse = await request.send();
      var response = await http.Response.fromStream(streamedResponse);

      responseJson = _returnResponse(response);
    } on SocketException {
      print('No net');
      throw FetchDataException('No Internet connection');
    }
    return responseJson;
  }

  String base64EncodeImage(String imageUri) {
    List<int> imageBytes = new File(imageUri).readAsBytesSync();
    return base64Encode(imageBytes);
  }

  @override
  Future<dynamic> uploadFace(KycDocument document) async {
    var responseJson;
    try {
      final response = await http.post(Uri.parse(this._url + "/sdk/face"),
          headers: <String, String>{
            'sdk-token': _sdkToken,
            'Content-Type': 'application/json'
          },
          body: json
              .encode({"selfie_image": base64EncodeImage(document.selphie)}));
      responseJson = _returnResponse(response);
    } on SocketException {
      print('No net');
      throw FetchDataException('No Internet connection');
    }
    return responseJson;
  }

  dynamic _returnResponse(http.Response response) {
    switch (response.statusCode) {
      case 201:
      case 200:
        var responseJson = json.decode(response.body.toString());
        print(responseJson);
        return responseJson;
      case 400:
        throw BadRequestException(response.body.toString());
      case 401:
      case 403:
        throw UnauthorisedException(response.body.toString());
      case 500:
      default:
        throw FetchDataException(
            'Error occured while Communication with Server with StatusCode : ${response.statusCode}');
    }
  }
}
