import 'dart:async';

import 'package:kyc/Components/document_type.dart';
import 'package:kyc/Exceptions/sdk_exceptions.dart';
import 'package:kyc/Helper/api_helper.dart';
import 'package:kyc/Helper/demo_api_helper.dart';
import 'package:kyc/Helper/iapi_helper.dart';
import 'package:kyc/Models/kyc_document.dart';
import 'package:kyc/Utils/channel_utils.dart';
import 'package:kyc/Utils/configuration_consts.dart';
import 'package:kyc/Utils/configuration_utils.dart';

class CallApiService {
  IApiHelper _apiHelper;
  int customerId = 1;
  ChannelUtils _channelUtils;

  CallApiService() {
    _channelUtils = new ChannelUtils();
  }

  CallApiService.withChannel(ChannelUtils channelUtils) {
    _channelUtils = channelUtils;
  }

  CallApiService.withChannelApiHelper(
      ChannelUtils channelUtils, ApiHelper apiHelper) {
    _channelUtils = channelUtils;
    _apiHelper = apiHelper;
  }

  static Future<CallApiService> create() async {
    CallApiService service = new CallApiService();
    await service.init();
    return service;
  }

  Future<void> init() async {
    String environment = await _channelUtils.getEnvironment();
    if (ConfigurationConsts.DEMO == environment) {
      _apiHelper = new DemoApiHelper();
    } else {
      _apiHelper = new ApiHelper(
          await ConfigurationUtils.getUrlForEnvironment(environment));
    }
  }

  Future<dynamic> sendDocument(KycDocument document, String country) async {
    String sdkToken = await _channelUtils.getSdkToken();
    if (sdkToken.isEmpty) {
      throw new SdkTokenMissingException();
    }
    _apiHelper.setSdkToken(sdkToken);

    if (document.type == DocumentType.FaceScan) {
      return await _apiHelper.uploadFace(document);
    } else {
      return await _apiHelper.uploadDocument(document, country);
    }
  }
}
