import 'package:flutter/services.dart';
import 'package:kyc/Components/document_type.dart';
import 'package:kyc/Exceptions/sdk_exceptions.dart';

class ChannelUtils {
  static const _PLATFORM_CLEARDIL =
      const MethodChannel('cleardil.sdk/kyc.channel');

  Future<String> getSdkToken() async {
    try {
      return await _PLATFORM_CLEARDIL.invokeMethod('getSdkToken');
    } on PlatformException catch (e) {
      print("Failed to get sdk_token: '${e.message}'.");
      throw new SdkTokenMissingException();
    }
  }

  Future<String> getEnvironment() async {
    try {
      return await _PLATFORM_CLEARDIL.invokeMethod('getEnvironment');
    } on PlatformException catch (e) {
      print("Failed to get url_environment: '${e.message}'.");
      throw new EnvironmentMissingException();
    }
  }

  static Future<List<DocumentType>> getListDocument() async {
    try {
      var types = List<DocumentType>.empty(growable: true);
      String listDocumentStr =
          await _PLATFORM_CLEARDIL.invokeMethod('getListDocuments');
      if (listDocumentStr != null && "" != listDocumentStr) {
        var docs = listDocumentStr.split(";");
        docs.forEach((element) {
          types.add(DocumentTypeStr.convert(element));
        });
      }
      return types;
    } on PlatformException catch (e) {
      print("Failed to get document list: '${e.message}'.");
      return DocumentType.values;
    } on MissingPluginException catch (e) {
      return DocumentType.values;
    }
  }
}
