import 'dart:io';

import 'package:camera/camera.dart';
import 'package:camera_platform_interface/camera_platform_interface.dart';
import 'package:device_info/device_info.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:kyc/Components/IdScan/scan_document.dart';
import 'package:kyc/Components/state.dart';
import 'package:provider/provider.dart';

import 'Components/Preview/document_preview.dart';
import 'Components/Welcome/welcome.dart';

Future<void> main() async {
  WidgetsFlutterBinding.ensureInitialized();
  SystemChrome.setPreferredOrientations(
          [DeviceOrientation.portraitUp])
      .then((_) => CleardilSdk.launchApp());
}

class CleardilSdk extends StatelessWidget {
  static Future<void> launchApp() async {
    CameraController camController;
    bool isPhysicalDevice = false;
    KycState kycState;
    if (Platform.isAndroid) {
      isPhysicalDevice =
          (await DeviceInfoPlugin().androidInfo).isPhysicalDevice;
    } else if (Platform.isIOS) {
      isPhysicalDevice = (await DeviceInfoPlugin().iosInfo).isPhysicalDevice;
    }
    if (!isPhysicalDevice) {
      kycState = KycState.simulator();
    } else {
        camController = await _initCamera();
      kycState = KycState.physical(camController);
    }
    runApp(MultiProvider(
      providers: [
        ChangeNotifierProvider(
          create: (_) => kycState,
        )
      ],
      child: CleardilSdk(),
    ));
  }

  static Future<CameraController> _initCamera() async {
    try {
      List<CameraDescription> cameras = await availableCameras();
      CameraDescription backCamera = cameras.firstWhere(
          (element) => element.lensDirection == CameraLensDirection.back);
      final CameraController cameraController = CameraController(
        backCamera,
        ResolutionPreset.medium,
        enableAudio: false,
      );
      await cameraController.initialize();
      await cameraController
          .lockCaptureOrientation(DeviceOrientation.portraitUp);
      return cameraController;
    } on CameraException catch (e) {
      print('Error: ${e.code}\nError Message: ${e.description}');
    }
  }

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      //TODO Each added screen must be added to the route
      routes: {
        WelcomeStep.routeName: (context) => WelcomeStep(),
        DocumentPreview.routeName: (context) => DocumentPreview(),
        ScanDocumentStep.routeName: (context) => ScanDocumentStep(),
      },
      title: 'KYC',
      debugShowCheckedModeBanner: false,
      theme: ThemeData(
          buttonColor: Colors.indigo,
          primarySwatch: Colors.blue,
          visualDensity: VisualDensity.adaptivePlatformDensity),
      home: Scaffold(
          body: Stack(
        fit: StackFit.expand,
        children: [Align(alignment: Alignment.topCenter, child: WelcomeStep())],
      )),
    );
  }
}
